﻿using System;
using System.Collections;
using System.IO;
using System.IO.IsolatedStorage;
using System.Windows;

namespace TestProject
{
    /// <summary>
    /// 메인 애플리케이션
    /// </summary>
    public partial class MainApplication : Application
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 애플리케이션 종료시 처리하기 - Application_Exit(sender, e)

        /// <summary>
        /// 애플리케이션 종료시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Application_Exit(object sender, ExitEventArgs e)
        {
            try
            {
                if(e.ApplicationExitCode == (int)ApplicationExitCode.Success)
                {
                    WriteApplicationLogEntry("Success", e.ApplicationExitCode);
                }
                else
                {
                    WriteApplicationLogEntry("Failure", e.ApplicationExitCode);
                }
            }
            catch
            {
                e.ApplicationExitCode = (int)ApplicationExitCode.CanNotWriteToApplicationLog;
            }

            try
            {
                WriteApplicationState();
            }
            catch
            {
                e.ApplicationExitCode = (int)ApplicationExitCode.CanNotWriteApplicationState;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 애플리케이션 로그 엔트리 쓰기 - WriteApplicationLogEntry(message, exitCode)

        /// <summary>
        /// 애플리케이션 로그 엔트리 쓰기
        /// </summary>
        /// <param name="message">메시지</param>
        /// <param name="exitCode">종료 코드</param>
        private void WriteApplicationLogEntry(string message, int exitCode)
        {
            IsolatedStorageFile file = IsolatedStorageFile.GetUserStoreForAssembly();

            using(IsolatedStorageFileStream stream = new IsolatedStorageFileStream("log.txt", FileMode.Append, FileAccess.Write, file))
            {
                using(StreamWriter writer = new StreamWriter(stream))
                {
                    string entry = string.Format
                    (
                    
                        "[{0}] : {1} - {2}",
                        DateTime.Now.ToString("HH:mm:dd"),
                        exitCode,
                        message
                    );

                    writer.WriteLine(entry);
                }
            }
        }

        #endregion
        #region 애플리케이션 상태 쓰기 - WriteApplicationState()

        /// <summary>
        /// 애플리케이션 상태 쓰기
        /// </summary>
        private void WriteApplicationState()
        {
            IsolatedStorageFile file = IsolatedStorageFile.GetUserStoreForAssembly();

            using(Stream stream = new IsolatedStorageFileStream("state.txt", FileMode.Create, file))
            {
                using(StreamWriter writer = new StreamWriter(stream))
                {
                    foreach(DictionaryEntry entry in Properties)
                    {
                        writer.WriteLine(entry.Value);
                    }
                }
            }
        }

        #endregion
    }
}